unit uregistros;

{$mode delphi}

interface

uses
  Classes, SysUtils, xmatdefs, matreal;


const
  camino = 'C:\basura\Florida1\';

type

  TRegistroSampleoUniforme = class
    idActor, idSerie: string;
    dt1: TDateTime;
    dts: TDateTime;
    v: TVectR;
    constructor Create(idActor, idSerie: string; dt1, dts: TDateTime; n: integer);
    constructor LoadFromArchi(archi: string);
    constructor LoadFromText(var f: textfile);
    procedure WriteToText(var f: textfile);
    procedure WriteToArchi(archi: string);
    procedure Free;
  end;

  TRegistroSampleoAsincrono = class
    idActor, idSerie: string;
    dt: TVectR;
    v: TVectR;
    constructor Create_init(
      idActor, idSerie: string; nMuestras: integer );
    constructor LoadFromArchi(
      idActor, idSerie, archi: string);
    function Resampling(dts: TDateTime): TRegistroSampleoUniforme;
    procedure Free;
  end;

  TTramosAsincronosLst = class( TList )
    idActor, idSerie: string;
    constructor Create( idActor, idSerie: string );
    procedure AddFromArchi( archi: string );
    procedure Unificar;
    procedure Free;
  end;

  TColeccionDeRegistros = class( TList )
    constructor Create;
    procedure WriteToArchi( archi: string );
  private
    n:Integer;
    kshift: TDAofNInt;
    dt_min, dts: TDateTime;

    function val( kSerie, kDato: integer ): NReal;
    procedure CalcShifts;
  end;

implementation


constructor TRegistroSampleoUniforme.Create(
  idActor, idSerie: string;
  dt1, dts: TDateTime; n: integer);
begin
  inherited Create;
  self.idActor:= idActor;
  self.idSerie := idSerie;
  self.dt1 := dt1;
  self.dts := dts;
  v := TVectR.Create_Init(n);
end;

constructor TRegistroSampleoUniforme.LoadFromText(var f: textfile);
var
  k, n: integer;
  a: NReal;
begin
  inherited Create;
  readln( f, idActor );
  readln(f, idSerie);
  readln(f, dt1);
  readln(f, dts);
  readln(f, n);
  v := TVectR.Create_init(n);
  for k := 1 to n do
  begin
    readln(f, a);
    v.pon_e(k, a);
  end;
end;

constructor TRegistroSampleoUniforme.LoadFromArchi(archi: string);
var
  f: textfile;
begin
  assignfile(f, archi);
  reset(f);
  LoadFromText(f);
  closefile(f);
end;

procedure TRegistroSampleoUniforme.WriteToText(var f: textfile);
var
  k, n: integer;
  a: NReal;
begin
  writeln( f, idActor );
  writeln(f, idSerie );
  writeln(f, dt1);
  writeln(f, dts);
  writeln(f, n);
  for k := 1 to n do
    writeln(f, v.e(k));
end;

procedure TRegistroSampleoUniforme.WriteToArchi(archi: string);
var
  f: textfile;
begin
  assignfile(f, archi);
  rewrite(f);
  WriteToText(f);
  closefile(f);
end;


procedure TRegistroSampleoUniforme.Free;
begin
  v.Free;
  inherited Free;
end;

constructor TRegistroSampleoAsincrono.Create_init(
  idActor, idSerie: string; nMuestras: integer );
begin
  inherited Create;
  self.idActor:= idActor;
  self.idSerie:= idSerie;
  dt:= TVectR.Create_Init( nMuestras );
  v:= TVectR.Create_Init( nMuestras );
end;

constructor TRegistroSampleoAsincrono.LoadFromArchi(
  idActor, idSerie, archi: string);
var
  f: textfile;
  cnt: integer;
  r: string;
  k, i: integer;
  dts: string;
  dtt: TDateTime;
  a: NReal;
  cnt_skip: integer;

begin
  inherited Create;
  self.idActor:= idActor;
  self.idSerie:= idSerie;

  filemode := 0;
  assignfile(f, camino+archi);
  reset(f);

  cnt := 0;
  while not EOF(f) do
  begin
    readln(f, r);
    if pos( 'Created', r ) = 0 then
      Inc(cnt);
  end;

  //   EFLOR1  .000 DIR VIENTO 85M 1    .AV 01/02/2015 02:01:26 49,78125
  dt := TVectR.Create_Init(cnt);
  v := TVectR.Create_INit(cnt);

  reset(f);
  k:= 1;
  cnt_skip:= 0;
  while not eof( f ) do
  begin
    readln(f, r);
    if pos( 'Created', r ) = 0 then
    begin
      i := pos('/', r);
      Delete(r, 1, i - 3);
      dts := copy(r, 1, 19);
      Delete(r, 1, 20);
      try
        dtt := StrToDateTime(dts);
        r := StringReplace(r, '.', '', [rfReplaceAll]);
        r := StringReplace(r, ',', '.', [rfReplaceAll]);
        r := trim(r);
        a := StrToFloat(r);
        dt.pon_e(k, dtt);
        v.pon_e(k, a);
        inc( k );
      except
        inc( cnt_skip );
      end;
    end;
  end;
  if cnt_skip >  0 then
  begin
    dt.resize( dt.n - cnt_skip );
    v.resize( v.n - cnt_skip );
  end;
  closefile(f);
end;

function TRegistroSampleoAsincrono.Resampling(dts: TDateTime):
TRegistroSampleoUniforme;
var
  res: TRegistroSampleoUniforme;
  dt1, dt2, dtt: TDateTime;
  n10min: integer;
  k: integer;
  kr, a: NReal;
begin
  dt1 := dt.e(1);
  dt2 := dt.e(dt.n);
  n10min := trunc((dt2 - dt1) / dts);
  dt1 := trunc(dt1 / dts) * dts;
  res := TRegistroSampleoUniforme.Create( idActor, idSerie, dt1, dts, n10min);
  for k := 1 to n10min do
  begin
    dtt := dt1 + (k - 1) * dts;
    kr := dt.inv_interpol(dtt);
    a := v.interpol(kr);
    res.v.pon_e(k, a);
  end;
  Result := res;
end;


procedure TRegistroSampleoAsincrono.Free;
begin
  dt.Free;
  v.Free;
  inherited Free;
end;


constructor TTramosAsincronosLst.Create( idActor, idSerie: string );
begin
  inherited Create;
  self.idActor:= idActor;
  self.idSerie:= idSerie;
end;

procedure TTramosAsincronosLst.AddFromArchi( archi: string );
var
  aReg: TRegistroSampleoAsincrono;
begin
  aReg:= TRegistroSampleoAsincrono.LoadFromArchi( idActor, idSerie, archi );
  if aReg.dt.n > 1 then
   add( aReg )
  else
    aReg.Free;
end;

function CompTramos(  Item1: Pointer;
  Item2: Pointer
):Integer;
var
   dif: double;
begin
   dif:= TRegistroSampleoAsincrono( item1 ).dt.e(1) - TRegistroSampleoAsincrono( item2 ).dt.e(1);
  if dif < 0  then
   result:= -1
  else if dif = 0 then
   result:= 0
  else
    result:= 1;
end;

procedure TTramosAsincronosLst.Unificar;
var
  k: integer;
  cnt_Solapamientos: integer;
  aTramo, bTramo: TRegistroSampleoAsincrono;
  dtMax: TDateTime;
  dtDif: TDateTime;

  RegUnico: TRegistroSampleoAsincrono;
  cnt_Muestras: integer;
  j, jj: integer;

begin
  if count = 0 then exit;

  Sort(  CompTramos );
  cnt_Solapamientos:= 0;
  cnt_Muestras:= 0;
  dtMax:= -1000;
  for k:= 0 to count-2 do
  begin
    aTramo:= items[k];
    cnt_Muestras:= cnt_Muestras + aTramo.dt.n;
    bTramo:= items[k+1];
    dtDif:= bTramo.dt.e( 1 ) - aTramo.dt.e( aTramo.dt.n );
    if dtDif < 0  then
       inc( cnt_Solapamientos );
    if dtDif > dtMax then
      dtMax:= dtDif;
  end;
  aTramo:= items[ count-1 ];
  cnt_Muestras:= cnt_Muestras + aTramo.dt.n;

  if cnt_Solapamientos > 0 then Raise Exception.Create('Hay solapamientos');

  jj:= 1;
  RegUnico:= TRegistroSampleoAsincrono.Create_init( idActor, idSerie, cnt_Muestras );
  for k:= 0 to count-1 do
   begin
     aTramo:= items[k];
     for j:= 1 to aTramo.dt.n do
      begin
        RegUnico.dt.pon_e( jj, aTramo.dt.e( j ) );
        RegUnico.v.pon_e( jj, aTramo.v.e( j ) );
        inc( jj );
      end;
     aTramo.Free;
   end;
   clear;
   add( RegUnico );

end;

procedure TTramosAsincronosLst.Free;
var
  aTramo: TRegistroSampleoAsincrono;
  k: integer;
begin
  for k:= 0 to count-1 do
   begin
     aTramo:= items[k];
     aTramo.Free;
   end;
  inherited Free;
end;

constructor TColeccionDeRegistros.Create;
begin
  inherited Create;
end;


procedure TColeccionDeRegistros.CalcShifts;

var
  k: integer;
  aSerie: TRegistroSampleoUniforme;
  dtFin, dt_max: TDateTime;

begin
  if count <= 0 then raise Exception.Create( 'CalcShifts llamado sin registros ' );
  aSerie:= items[0];
  self.dts:= aSerie.dts;
  dt_min:= aSerie.dt1;
  dt_max:= dt_min + (aSerie.v.n-1) * dts;

  for k:= 1 to count -1 do
   begin
     aSerie:= items[k];
     if aSerie.dts <> dts then raise Exception.Create('OJO no coinciden los dts de las series.');
     if aSerie.dt1 < dt_min then
       dt_min:= aSerie.dt1;
     dtFin:=  aSerie.dt1 + (aSerie.v.n-1) * dts;
     if dtFin > dt_max then
       dt_max:= dtFin;
   end;
  n:= trunc( (dt_max - dt_min)/dts + 1 + 0.01 );
  setlength( kshift, count );
  for k:= 0 to count -1 do
   begin
     aSerie:= items[k];
      kshift[k]:= trunc( ( aSerie.dt1 - dt_min)/dts  + 0.01 );
  end;
end;

function TColeccionDeRegistros.val( kSerie, kDato: integer ): NReal;
var
  k: integer;
  aSerie: TRegistroSampleoUniforme;
begin
  aSerie:= items[kSerie];
  k:= kDato - kshift[ kSerie ];
  if k < 1 then
    result:= -1111110
  else if k <= aSerie.v.n then
    result:= aSerie.v.e(k )
  else
    result:= -1111111;
end;

procedure TColeccionDeRegistros.WriteToArchi( archi: string );
var
  f: textfile;
  aSerie: TRegistroSampleoUniforme;
  dt: TDateTime;
  kPaso: integer;
  kSerie: integer;

begin
  CalcShifts;

  assignfile( f, archi );
  rewrite( f );

  write( f, '--' );
  for kSerie:= 0 to count-1 do
  begin
    aSerie:= items[kSerie];
    write( f, #9, aSerie.idActor );
  end;
  writeln( f );

  write( f, 'dt' );
  for kSerie:= 0 to count-1 do
  begin
    aSerie:= items[kSerie];
    write( f, #9, aSerie.idSerie );
  end;
  writeln( f );

  for kPaso := 1 to n do
  begin
    dt:= dt_min + (kPaso-1) * dts;
    write( f, DateTimeToStr( dt ) );
    for kSerie:= 0 to count-1 do
      write( f, #9, val( kserie, kPaso ) );
    writeln( f );
  end;

  closefile( f );
end;

end.
