unit udemandas01;

interface
uses
	 Sysutils,
	 Math,
	 uauxiliares,
	 xmatdefs,
	 uglobs,
   uActorNodal,
   udemandas, uFuentesAleatorias,
   uNodos, usimplex, ufichasLPD,
	 ufechas,
	 ucosa,
	 uconstantesSimSEE, uCosaConNombre;

type
	TFichaDemanda01 = class(TFichaLPD)
    public
  		PotDiaHabil, PotDiaMedioFeriado, PotDiaFeriado: TDAOfNreal; // Potencia en MW
	  	constructor Create(
		  		fecha: TFecha; periodicidad : TPeriodicidad ;
			  	PotDiaHabil, PotDiaMedioFeriado, PotDiaFeriado: TDAOfNreal // Potencia en MW para cada hora de 0..23
    			);
		  constructor Create_ReadFromText( f: TArchiTexto ); override;
  		procedure WriteToText( f: TArchiTexto ); override;
      function infoAd : String; override;
	  	procedure Free; override;
	end;


type
	TDemanda01 = class( TDemanda )
		public
  		lpd : TFichasLPD;
	  	pA, pB: TFichaDemanda01; // Fichas de parmetros dinmicos

  		constructor Create(
	  		nombre: string;
		  	nacimiento, muerte: TFecha;
  			nodo: TNodo;
	  		pa_ini: TFichaDemanda01; // puede ser nil
		  	falla_profundidad, falla_costo: TDAOfNReal;
        fuente: TFuenteAleatoria;
        nombreBorne: String;
         icf_Fuente: TFuenteAleatoria;
         icf_NombreBorne: String );

  		constructor Create_ReadFromText( f: TArchiTexto ); override;
	  	procedure WriteToText( f: TArchiTexto ); override;
  		procedure RegistrarParametrosDinamicos; override;

		  function InfoAd : String; override;
  		class function DescClase : String; override;

      function PotenciaFirme : NReal; override;

//		  procedure prepararSim; override;
  		procedure prepararPaso_as; override;
		  procedure Free; override;
  	private
	  	CacheActivo: boolean;
  	end;


(* Deforma una curva de potencia horaria aplicando factores de crecimiento de
energa (fce) y del pico (fcp).
Por ejemplo, si el pico decrese 10% y la energa debe aumentar 5% los factores
sern fce= 1.05 y fcp= 0.9

El resultado es TRUE si todas las potencias resultantes son positivas y false
si alguna qued negativa.
*)
function aplicarCrecimientos_CurvaHoraria( var PotHoraria: TDAOfNReal;	fce, fcp: NReal ): boolean;

procedure AlInicio;
procedure AlFinal;

implementation


//----------------------
// Mtodos de TDemanda01
//======================

constructor TDemanda01.Create(
	 nombre: string;
	 nacimiento, muerte: TFecha;
	 nodo: TNodo;
	 pa_ini: TFichaDemanda01;
	 falla_profundidad, falla_costo: TDAOfNReal;
   fuente: TFuenteAleatoria;
   nombreBorne: String;
   icf_Fuente: TFuenteAleatoria;
   icf_NombreBorne: String );
begin
	inherited Create( nombre, nacimiento, muerte, nodo,
                falla_profundidad, falla_costo, fuente, nombreBorne,
                icf_fuente, icf_NombreBorne );


	self.lpd := TFichasLPD.Create(self, TFichaDemanda01);

  self.fuente:= fuente;
  self.nombreBorne:= nombreBorne;

	pa:= pa_ini;
	if pa <> nil then
		 self.lpd.Add(pa);

	CacheActivo:= false;
end;

constructor TDemanda01.Create_ReadFromText( f: TArchiTexto );
begin
	inherited Create_ReadFromText( f );
  f.rd('lpd', TCosa(lpd));
  f.EjecutarLectura;
  pa:= nil;
  CacheActivo:= false;
  lpd.Propietario:= self;
end;

procedure TDemanda01.WriteToText( f: TArchiTexto );
begin
	inherited WriteToText( f );
	f.wr('lpd', lpd);
end;


procedure TDemanda01.registrarParametrosDinamicos;
begin
	inherited registrarParametrosDinamicos;
 	lpd.expandirFichas(globs);
 	lpd.RegistrarFichasAActualizar(Self, globs.ActualizadorLPD, @pA, @pB );
end;

function TDemanda01.infoAd : String;
begin
	result := '';
end;

class function TDemanda01.DescClase : String;
begin
	result := '3 Curvas Horarias';
end;

function TDemanda01.potenciaFirme : NReal;
begin
	result := paUnidades.nUnidades * vmax(self.PPa);
end;

procedure TDemanda01.prepararPaso_as;
var
	alfa: NReal;

	hdd: integer;
	instante: TFecha;
	tipoDeDia: integer;
	iHoraDelPaso: integer;
begin
{	if CacheActivo then exit;
	CacheActivo:= true;}


	instante:= TFecha.Create_Clone(globs.FechaInicioDelpaso );

	alfa:= pA.fecha.HorasHasta( Instante )/ pA.fecha.HorasHasta( pB.fecha );
	hdd:= Instante.hora;

	for iHoraDelPaso:= 0 to high( PHoraria ) do
	begin
		tipoDeDia:= instante.TipoDeDia;
		case tipoDeDia of
			0: PHoraria[iHoraDelPaso]:=	ponder( pA.PotDiaHabil[hdd], pB.PotDiaHabil[hdd], alfa );
			1: PHoraria[iHoraDelPaso]:=	ponder( pA.PotDiaMedioFeriado[hdd], pB.PotDiaMedioFeriado[hdd], alfa );
			2: PHoraria[iHoraDelPaso]:=	ponder( pA.PotDiaFeriado[hdd], pB.PotDiaFeriado[hdd], alfa );
		end; // del case
		hdd:= ( hdd + 1 ) mod 24;
	end;
  inherited prepararPaso_as;
end;


procedure TDemanda01.free;
var
  i: Integer;
begin
	setlength( PPa, 0 );
	self.lpd.Free;

  for i:= 0 to High(fallas) do
    SetLength(fallas[i], 0);
  SetLength(fallas, 0);

  SetLength(falla_profundidad, 0);
  SetLength( falla_costo_0, 0);
  SetLength( falla_costo_indexado, 0 );

	inherited Free;
end;

//-------------------------
// Mtodos de TFichaDemanda
//=========================

constructor TFichaDemanda01.Create( fecha: TFecha; periodicidad : TPeriodicidad ;
		PotDiaHabil, PotDiaMedioFeriado, PotDiaFeriado: TDAOfNreal);
begin
	inherited Create( fecha, periodicidad );
	self.PotDiaHabil := copy( PotDiaHabil, 0 , MAXINT);
	self.PotDiaMedioFeriado := copy( PotDiaMedioFeriado, 0 , MAXINT );
	self.PotDiaFeriado:= copy( PotDiaFeriado, 0 , MAXINT );
end;


function aplicarCrecimientos_CurvaHoraria( var PotHoraria: TDAOfNReal;	fce, fcp: NReal ): boolean;
var
	k, posPico: integer;
	gamma: NReal;
	E, Pp, Pk, PpPrima: NReal;
	res: boolean;
	Pm: NReal;
begin
	E:= 0;
	Pp:= PotHoraria[0];
	posPico := 0;
	for k:= 0 to high( POtHOraria ) do
	begin
		Pk:=PotHoraria[k];
		E:= E+ Pk;
		if Pk > Pp then
			begin
			Pp:= Pk;
			posPico := k;
			end;
	end;
	Pm:= E/ length( PotHoraria );

	gamma:= (fcp * Pp - fce * Pm ) / ( Pp - Pm );

	res:= true;
	PpPrima := fcp * Pp;
	for k:= 0 to posPico - 1 do
	begin
		Pk := PpPrima - gamma * ( PP - PotHoraria[k]);
		if Pk < 0 then res:= false; // ojo, dio una potencia negativa
		PotHoraria[k]:= Pk;
	end;

	PotHoraria[posPico] := PpPrima;

	for k := posPico + 1 to high(PotHoraria) do
		begin
		Pk := PpPrima - gamma * ( PP - PotHoraria[k]);
		if Pk < 0 then res:= false; // ojo, dio una potencia negativa
		PotHoraria[k]:= Pk;
		end;

	result:= res;
end;

constructor TFichaDemanda01.Create_ReadFromText( f: TArchiTexto );
begin
	inherited Create_ReadFromText( f );
  f.IniciarLecturaRetrasada;
	f.rd( 'PotDiaHabil', PotDiaHabil);
	f.rd( 'PotDiaMedioFeriado', PotDiaMedioFeriado );
	f.rd( 'PotDiaFeriado', PotDiaFeriado );
  f.EjecutarLectura;
end;


procedure TFichaDemanda01.WriteToText( f: TArchiTexto );
begin
	inherited WriteToText( f );
	f.wr('PotDiaHabil', PotDiaHabil, uconstantesSimSEE.CF_PRECISION, uconstantesSimSEE.CF_DECIMALES );
	f.wr('PotDiaMedioFeriado', PotDiaMedioFeriado, uconstantesSimSEE.CF_PRECISION, uconstantesSimSEE.CF_DECIMALES );
	f.wr('PotDiaFeriado', PotDiaFeriado, uconstantesSimSEE.CF_PRECISION, uconstantesSimSEE.CF_DECIMALES );
end;

function TFichaDemanda01.infoAd : String;
begin
  result:= 'PotDaHbil Media= ' + FloatToStrF(vprom(PotDiaHabil), ffGeneral, 10, 1) + ' MW, ' +
           'PotDaMedioFeriado Media= ' + FloatToStrF(vprom(PotDiaMedioFeriado), ffGeneral, 10, 1) + ' MW, ' +
           'PotDaFeriado Media= ' + FloatToStrF(vprom(PotDiaFeriado), ffGeneral, 10, 1) + ' MW';
end;

procedure TFichaDemanda01.free;
begin
	setlength( PotDiaHabil, 0 );
	setlength( PotDiaMedioFeriado, 0 );
	setlength( PotDiaFeriado, 0 );
	inherited Free;
end;



procedure AlInicio;
begin
	registrarClaseDeCosa( TDemanda01.ClassName, TDemanda01 );
	registrarClaseDeCosa( TFichaDemanda01.ClassName, TFichaDemanda01 );
end;
procedure AlFinal;
begin
end;

end.
