﻿unit uActorNodalCombustible;

{$IFDEF FPC}
  {$MODE Delphi}
{$ENDIF}

interface

uses
  uCosa, uGlobs, uActores, uFechas, unodoCombustible, uCosaConNombre, uFichasLPD,
  xMatDefs, uauxiliares, uConversions;

type

  { TActorNodalCombustible }

  TActorNodalCombustible = class(TActor)
    function ReferenciaAlNodoCombustible (nodocomb: TNodoCombustible): boolean; virtual; abstract;
  end;

  //TActorUniNodalCombustible es un nodo NO ELÉCTRICO

  TActorUniNodalCombustible = class(TActorNodalCombustible)

  public
    (**************************************************************************)
    (*               A T R I B U T O S   P E R S I S T E N T E S              *)
    (**************************************************************************)

    NodoComb: TNodoCombustible;
    (**************************************************************************)
    Q: TDAOfNReal;   // caudal inyectado al nodo

    constructor Create(capa: integer; nombre: string; nacimiento, muerte: TFecha;
      lpdUnidades: TFichasLPD; nodocomb: TNodoCombustible);
    {$IFNDEF MIGRACION_PERSISTENCIA}
    constructor Create_ReadFromText(f: TArchiTexto); override;
    procedure WriteToText(f: TArchiTexto); override;
    {$ENDIF}
    function ReferenciaAlNodoCombustible (nodocomb: TNodoCombustible): boolean; override;
    procedure PubliVars; override;
    procedure PrepararMemoria(globs: TGlobs); override;
    procedure Free; override;

    {$IFDEF MIGRACION_PERSISTENCIA}
    class function CreateDataColumnList(xClaseDeCosa:TClaseDeCosa; xVersion: Integer=-2): TDataColumnListOfCosa; override;
    published
      property _NodoComb: TNodoCombustible read NodoComb write NodoComb;
    {$ENDIF}

  end;

  { TActorBiNodalCombustible }

  TActorBiNodalCombustible = class(TActorNodalCombustible)

  public

    (**************************************************************************)
    (*               A T R I B U T O S   P E R S I S T E N T E S              *)
    (**************************************************************************)

    NodoCombA, NodoCombB: TNodoCombustible;
    (**************************************************************************)

    Q_NodoCombA, Q_NodoCombB: TDAOfNReal;

    constructor Create(capa: integer; nombre: string; nacimiento, muerte: TFecha;
      lpdUnidades: TFichasLPD; NodoCombA, NodoCombB: TNodoCombustible);
    {$IFNDEF MIGRACION_PERSISTENCIA}
    constructor Create_ReadFromText(f: TArchiTexto); override;
    procedure WriteToText(f: TArchiTexto); override;
    {$ENDIF}
    function ReferenciaAlNodoCombustible (nodocomb: TNodoCombustible): boolean; override;
    procedure PrepararMemoria(globs: TGlobs); override;
    procedure Free; override;

    {$IFDEF MIGRACION_PERSISTENCIA}
    class function CreateDataColumnList(xClaseDeCosa:TClaseDeCosa; xVersion: Integer=-2): TDataColumnListOfCosa; override;
    class function CreateDataConversionList: TListDataConversion; override;
    {$ENDIF}

    procedure AfterInstantiation; override;

    published

      {$IFDEF MIGRACION_PERSISTENCIA}
      property _NodoCombA: TNodoCombustible read NodoCombA write NodoCombA;
      property _NodoCombB: TNodoCombustible read NodoCombB write NodoCombB;
      {$ENDIF}


  end;

procedure AlInicio;
procedure ALFinal;

implementation

{ TActorNodalCombustible }

//--------------------------
// Métodos de TActorUniNodalCombustible
//==========================

constructor TActorUniNodalCombustible.Create(capa: integer; nombre: string;
  nacimiento, muerte: TFecha; lpdUnidades: TFichasLPD;
  nodocomb: TNodoCombustible);
begin
  inherited Create(capa, nombre, nacimiento, muerte, lpdUnidades);
  self.NodoComb := NodoComb;
end;

{$IFNDEF MIGRACION_PERSISTENCIA}
constructor TActorUniNodalCombustible.Create_ReadFromText(f: TArchiTexto);
begin
  inherited Create_ReadFromText(f);
  f.IniciarLecturaRetrasada;
  f.rdReferencia('NodoComb', TCosa(NodoComb), Self);
  f.EjecutarLectura;
end;

procedure TActorUniNodalCombustible.WriteToText(f: TArchiTexto);
begin
  inherited WriteToText(f);
  f.wrReferencia('NodoComb', NodoComb);
end;
{$ENDIF}

function TActorUniNodalCombustible.ReferenciaAlNodoCombustible(
  nodocomb: TNodoCombustible): boolean;
begin
  Result := self.nodocomb = nodocomb;
end;

procedure TActorUniNodalCombustible.PubliVars;
begin
  inherited PubliVars;
  PublicarVariableVR('Q', '[m3/s]', 6, 2, Q, True, True);   //verificar unidades
end;

procedure TActorUniNodalCombustible.PrepararMemoria(globs: TGlobs);
begin
  inherited PrepararMemoria(globs);
  setlength(Q, globs.NPostes);
end;


procedure TActorUniNodalCombustible.Free;
begin
  setlength(Q, 0);
  inherited Free;
end;

{$IFDEF MIGRACION_PERSISTENCIA}
class function TActorUniNodalCombustible.CreateDataColumnList(
  xClaseDeCosa: TClaseDeCosa; xVersion: Integer): TDataColumnListOfCosa;
begin
  Result:=inherited CreateDataColumnList(xClaseDeCosa,xVersion);
  Result.AddCosaColumn('NodoComb');
end;
{$ENDIF}

//-------------------------
// Métodos de TActorBiNodalCombustible
//=========================

constructor TActorBiNodalCombustible.Create(capa: integer; nombre: string; nacimiento, muerte: TFecha;
  lpdUnidades: TFichasLPD; NodoCombA, NodoCombB: TNodoCombustible);
begin
  inherited Create(capa, nombre, nacimiento, muerte, lpdUnidades);
  self.NodoCombA := NodoCombA;
  self.NodoCombB := NodoCombB;
end;

{$IFNDEF MIGRACION_PERSISTENCIA}
constructor TActorBiNodalCombustible.Create_ReadFromText(f: TArchiTexto);
begin
  inherited Create_ReadFromText(f);
  f.IniciarLecturaRetrasada;
  f.rdReferencia('nodo combustible A', TCosa(NodoCombA), Self);
  f.rdReferencia('nodo combustible B', TCosa(NodoCombB), Self);
  f.EjecutarLectura;

end;

procedure TActorBiNodalCombustible.WriteToText(f: TArchiTexto);
begin
  inherited WriteToText(f);
  f.wrReferencia('nodo combustible A', NodoCombA);
  f.wrReferencia('nodo combustible B', NodoCombB);
end;
{$ENDIF}

function TActorBiNodalCombustible.ReferenciaAlNodoCombustible(
  nodocomb: TNodoCombustible): boolean;
begin
  Result := (nodoCombA = nodocomb) or (nodoCombB = nodocomb);
end;

procedure TActorBiNodalCombustible.PrepararMemoria(globs: TGlobs);
begin
  inherited prepararMemoria(globs);
  setlength(Q_NodoCombA, globs.NPostes);
  setlength(Q_NodoCombB, globs.NPostes);
end;

procedure TActorBiNodalCombustible.Free;
begin
  setlength(Q_NodoCombA, 0);
  setlength(Q_NodoCombB, 0);
  inherited Free;
end;

{$IFDEF MIGRACION_PERSISTENCIA}
class function TActorBiNodalCombustible.CreateDataColumnList(
  xClaseDeCosa: TClaseDeCosa; xVersion: Integer): TDataColumnListOfCosa;
begin

  Result:=inherited CreateDataColumnList(xClaseDeCosa,xVersion);
  Result.AddCosaColumn('nodo combustible A', 0, uConversions.VERSION_MIGRACION_PERSISTENCIA);
  Result.AddCosaColumn('nodo combustible B', 0, uConversions.VERSION_MIGRACION_PERSISTENCIA);

  Result.AddCosaColumn('NodoCombA', uConversions.VERSION_MIGRACION_PERSISTENCIA);
  Result.AddCosaColumn('NodoCombB', uConversions.VERSION_MIGRACION_PERSISTENCIA);

end;

class function TActorBiNodalCombustible.CreateDataConversionList: TListDataConversion;
begin
  Result:=inherited CreateDataConversionList;
  Result.Add(uConversions.VERSION_MIGRACION_PERSISTENCIA, ['nodo combustible A', 'nodo combustible B'],
    ['NodoCombA', 'NodoCombB'], Result.ConversionNAMECHANGE);
end;
{$ENDIF}

procedure TActorBiNodalCombustible.AfterInstantiation;
begin
  inherited AfterInstantiation;
end;

procedure AlInicio;
begin
  registrarClaseDeCosa(TActorUniNodalCombustible.ClassName, TActorUniNodalCombustible);
  registrarClaseDeCosa(TActorBiNodalCombustible.ClassName, TActorBiNodalCombustible);

end;

procedure AlFinal;
begin
end;

end.
