unit uforzamientos;


interface

uses
  Classes, SysUtils, xmatdefs, ufichasLPD, ucosa,
  ufechas,uconversions;



type

{ TFichaForzamientos }

  TFichaForzamientos = class(TFichaLPD)
  public
    activar_forzamiento: boolean;
    P: TDAofNReal;

    constructor Create(capa: integer; fecha: TFecha; periodicidad: TPeriodicidad;
      activar_forzamiento: boolean; P: TDAOfNReal);
    procedure Free; override;

    {$IFNDEF MIGRACION_PERSISTENCIA}
    constructor Create_ReadFromText(f: TArchiTexto); override;
    procedure WriteToText(f: TArchiTexto); override;
    {$ENDIF}

    function InfoAd_: string; override;
    procedure generarLineaResumen(var archi: TextFile); override;

    {$IFDEF MIGRACION_PERSISTENCIA}
    class function CreateDataColumnList(xClaseDeCosa:TClaseDeCosa; xVersion: Integer = -2): TDataColumnListOfCosa; override;
    class function CreateDataConversionList: TListDataConversion; override;
{$ENDIF}

  published
    {$IFDEF MIGRACION_PERSISTENCIA}
    property _activar_forzamiento: boolean     read activar_forzamiento write activar_forzamiento;
    property _P:                   TDAofNReal  read P                   write P;
    {$ENDIF}

  end;


procedure AlInicio;
procedure AlFinal;

resourcestring
  rsForzamientoInactivo = 'Inactivo';
  rsForzamientoActivo = 'Activo';


implementation

//-------------------------
//Metodos de TFichaForzamientos
//=========================
constructor TFichaForzamientos.Create(capa: integer; fecha: TFecha; periodicidad: TPeriodicidad;
     activar_forzamiento: boolean; P: TDAOfNReal);
begin
  inherited Create(capa, fecha, periodicidad);
  self.activar_forzamiento:= activar_forzamiento;
  self.P:= P;
end;

procedure TFichaForzamientos.Free;
begin
  setlength(P, 0);
  inherited Free;
end;


{$IFNDEF MIGRACION_PERSISTENCIA}
constructor TFichaForzamientos.Create_ReadFromText(f: TArchiTexto);
begin
  inherited Create_ReadFromText(f);
  f.IniciarLecturaRetrasada;
  f.rd('activo', activar_forzamiento );
  f.rd('P', P);
  f.EjecutarLectura;
end;
{$ENDIF}

{$IFNDEF MIGRACION_PERSISTENCIA}
procedure TFichaForzamientos.WriteToText(f: TArchiTexto);
begin
  inherited WriteToText(f);
  f.wr('activo', activar_forzamiento );
  f.wr('P', P);
end;
{$ENDIF}

function TFichaForzamientos.InfoAd_: string;
begin
  if activar_forzamiento then
   Result := inherited infoAd_ +rsForzamientoActivo+', '+ DAOfNRealToStr_(self.P, 12, 2, ';')
  else
   Result := inherited infoAd_ +rsForzamientoInactivo+', '+ DAOfNRealToStr_(self.P, 12, 2, ';')
end;

procedure TFichaForzamientos.generarLineaResumen(var archi: TextFile);
begin
  Write(archi, infoAd_, #9);  //NUnidades
end;

{$IFDEF MIGRACION_PERSISTENCIA}
class function TFichaForzamientos.CreateDataColumnList(xClaseDeCosa:TClaseDeCosa; xVersion: Integer
  ): TDataColumnListOfCosa;
begin
  Result:=inherited CreateDataColumnList(xClaseDeCosa, xVersion);
  Result.AddBooleanColumn   ('activo', true,0,VERSION_MIGRACION_PERSISTENCIA);
  Result.AddBooleanColumn   ('activar_forzamiento', true,VERSION_MIGRACION_PERSISTENCIA);
  Result.AddDAOfDoubleColumn('P');
end;

class function TFichaForzamientos.CreateDataConversionList: TListDataConversion;
begin
  Result:=inherited CreateDataConversionList;
  Result.Add(VERSION_MIGRACION_PERSISTENCIA,['activo'],['activar_forzamiento'],Result.ConversionNAMECHANGE);
end;
{$ENDIF}

procedure AlInicio;
begin
  registrarClaseDeCosa(TFichaForzamientos.ClassName, TFichaForzamientos);
end;

procedure AlFinal;
begin
end;


end.

