unit ufuenteselector_horario;

interface

(*+doc
Unidad: ufuenteselector_horario
La idea es que la fuente tenga un conjunto de fuentes de entrada cada una con un filtro
de las horas del día en que será seleccionada para ser copiada a la salida.
-doc*)

uses
  Classes, SysUtils,
  uFuentesAleatorias, ucosa,
  uFichasLPD, xMatDefs, uFechas,
  uauxiliares,
  uglobs;

resourcestring
  rsFichaDe = 'Ficha de';
  rsFuenteSelector_horario = 'Fuente selector horario';

type

  { TFuenteYHorario }

  TFuenteYHorario = class(TCosa)
  public
    (**************************************************************************)
    (* A T R I B U T O S   P E R S I S T E N T E S                            *)
    (**************************************************************************)
    fuente: TFuenteAleatoria;
    idBorne: integer;
    // suponemos un array ordenado de horas por ej {2,4,23}
    horario_FERIADO: TDAOfNInt;
    horario_SEMI_FERIADO: TDAOfNInt;
    horario_HABIL: TDAOfNInt;
    (**************************************************************************)

    constructor Create(xFuente: TFuenteAleatoria; xidBorne: integer; xhorario_HABIL, xhorario_SEMI_FERIADO, xhorario_FERIADO: TDAOfNInt);

    {$IFNDEF MIGRACION_PERSISTENCIA}
    constructor Create_ReadFromText(f: TArchiTexto); override;
    procedure WriteToText(f: TArchiTexto); override;
    {$ENDIF}

    function en_horario(h: integer; tipoDeDia: TTipoDeDia): boolean;
    {$IFDEF MIGRACION_PERSISTENCIA}
    class function CreateDataColumnList(xClaseDeCosa:TClaseDeCosa; xVersion: Integer = -2): TDataColumnListOfCosa; virtual;
    {$ENDIF}
    procedure AfterInstantiation; override;
  published
    {$IFDEF MIGRACION_PERSISTENCIA}
    property _fuente:               TFuenteAleatoria read fuente               write fuente;
    property _idBorne:              integer          read idBorne              write idBorne;
    property _horario_FERIADO:      TDAOfNInt        read horario_FERIADO      write horario_FERIADO;
    property _horario_SEMI_FERIADO: TDAOfNInt        read horario_SEMI_FERIADO write horario_SEMI_FERIADO;
    property _horario_HABIL:        TDAOfNInt        read horario_HABIL        write horario_HABIL;
    {$ENDIF}
  end;

(*+doc TFichaFuenteSelector_horario
Ficha de parámetros dinámicos de las fuentes Selector
Permite especificar fuentes (y bornes) de entrada
-doc*)

  { TFichaFuenteSelector_horario }

  TFichaFuenteSelector_horario = class(TFichaLPD)
  public
    (**************************************************************************)
    (* A T R I B U T O S   P E R S I S T E N T E S                            *)
    (**************************************************************************)
    usarTipoDeDia:                boolean;
    FuentesYHorarios:             TListaDeCosas;
    ValorPorDefecto_HABIL:        NReal;
    ValorPorDefecto_SEMI_FERIADO: NReal;
    ValorPorDefecto_FERIADO:      NReal;
    (**************************************************************************)

    constructor Create(capa: integer; fecha: TFecha; periodicidad: TPeriodicidad;
      xusarTipoDeDia: boolean; fuentesYHorarios: TListaDeCosas;
  ValorPorDefecto_HABIL, ValorPorDefecto_SEMI_FERIADO,
  ValorPorDefecto_FERIADO: NReal);
    {$IFNDEF MIGRACION_PERSISTENCIA}
    constructor Create_ReadFromText(f: TArchiTexto); override;
    procedure WriteToText(f: TArchiTexto); override;
    {$ENDIF}
    class function DescClase: string; override;
    function infoAd_: string; override;

    {$IFDEF MIGRACION_PERSISTENCIA}
    class function CreateDataColumnList(xClaseDeCosa:TClaseDeCosa; xVersion: Integer = -2): TDataColumnListOfCosa; virtual;
    {$ENDIF}
  published
    {$IFDEF MIGRACION_PERSISTENCIA}
    property _usarTipoDeDia:                boolean       read usarTipoDeDia                write usarTipoDeDia;
    property _FuentesYHorarios:             TListaDeCosas read FuentesYHorarios             write FuentesYHorarios;
    property _ValorPorDefecto_HABIL:        NReal         read ValorPorDefecto_HABIL        write ValorPorDefecto_HABIL;
    property _ValorPorDefecto_SEMI_FERIADO: NReal         read ValorPorDefecto_SEMI_FERIADO write ValorPorDefecto_SEMI_FERIADO;
    property _ValorPorDefecto_FERIADO:      NReal         read ValorPorDefecto_FERIADO      write ValorPorDefecto_FERIADO;
    {$ENDIF}

  end;

(*+doc TFuenteSelector_horario
Según la hora de inicio del paso selecciona una de las fuentes de entrada
como salida.
-doc*)
  TFuenteSelector_horario = class(TFuenteAleatoriaConFichas)
    pa: TFichaFuenteSelector_horario;

    constructor Create(capa: integer; nombre: string; lpd: TFichasLPD);
    class function TipoFichaFuente: TClaseDeFichaLPD; override;

    //function InfoAd : String; override;
    class function DescClase: string; override;
    procedure RegistrarParametrosDinamicos; override;

    procedure PrepararPaso_ps; override;
    procedure SortearEntradaRB(var aRB: NReal); override;
    procedure ValorEsperadoEntradaRB(var aRB: Nreal); override;
    function referenciaFuente(fuente: TFuenteAleatoria): boolean; override;
  end;

procedure procCambioFichaSelector_horario(fuente: TCosa);
procedure AlInicio;
procedure AlFinal;

implementation


constructor TFuenteYHorario.Create(xFuente: TFuenteAleatoria;
  xidBorne: integer; xhorario_HABIL, xhorario_SEMI_FERIADO,
  xhorario_FERIADO: TDAOfNInt);
begin
  fuente := xFuente;
  idBorne := xidBorne;
  horario_FERIADO := xhorario_FERIADO;
  horario_SEMI_FERIADO := xhorario_SEMI_FERIADO;
  horario_HABIL := xhorario_HABIL;
end;

{$IFNDEF MIGRACION_PERSISTENCIA}
constructor TFuenteYHorario.Create_ReadFromText(f: TArchiTexto);
begin
  inherited Create_ReadFromText(f);
  f.rdReferencia('fuente', TCosa(fuente), self);
  f.rd('idBorne', idBorne);
  f.rd('horario_HABIL', horario_HABIL);
  f.rd('horario_SEMI_FERIADO', horario_SEMI_FERIADO);
  f.rd('horario_FERIADO', horario_FERIADO);
  f.EjecutarLectura;

// obligo que los horarios esté ordenados.
  QuickSort_Creciente( horario_HABIL);
  QuickSort_Creciente( horario_SEMI_FERIADO );
  QuickSort_Creciente( horario_FERIADO );
end;
{$ENDIF}

{$IFNDEF MIGRACION_PERSISTENCIA}
procedure TFuenteYHorario.WriteToText(f: TArchiTexto);
begin
  inherited WriteToText(f);
  f.wrReferencia('fuente', TCosa(fuente));
  f.wr('idBorne', idBorne);
  f.wr('horario_HABIL', horario_HABIL);
  f.wr('horario_SEMI_FERIADO', horario_SEMI_FERIADO);
  f.wr('horario_FERIADO', horario_FERIADO);
end;
{$ENDIF}

function TFuenteYHorario.en_horario(h: integer; tipoDeDia: TTipoDeDia): boolean;
var
  k1, k2, k: integer;
  buscando: boolean;
  horario: TDAOfNInt;
begin

  case TipoDeDia of
    DIA_HABIL: horario := horario_HABIL;
    DIA_SEMI_FERIADO: horario := horario_SEMI_FERIADO;
    DIA_FERIADO: horario := horario_FERIADO;
  end;

  if length( horario ) = 0 then
  begin
    result:= false;
    exit;
  end;

  // busqueda rápida en array ordenado
  buscando := True;
  k1 := 0;
  k2 := high(horario);
  buscando := true;

// writeln( 'TipoD: ', Ord(TipoDeDia), ', k1: ', k1, ', k2: ', k2 );
  if ( horario[k1] <= h ) and (h  <=  horario[k2] ) then
    while buscando and ( k2 >= k1 ) do
    begin
      k := (k1 + k2) div 2;
      if horario[k] = h then
        buscando := False
      else
        if horario[k] > h then
          k2 := k-1
        else
          k1 := k+1;
    end;

  Result := not buscando;
end;

{$IFDEF MIGRACION_PERSISTENCIA}
class function TFuenteYHorario.CreateDataColumnList(xClaseDeCosa: TClaseDeCosa;
  xVersion: Integer): TDataColumnListOfCosa;
begin
  Result:=inherited CreateDataColumnList(xClaseDeCosa,xVersion);
  Result.AddCosaColumn   ('fuente');
  Result.AddIntegerColumn('idBorne',              0);
  Result.AddIntegerColumn('horario_HABIL',        0);
  Result.AddIntegerColumn('horario_SEMI_FERIADO', 0);
  Result.AddIntegerColumn('horario_FERIADO',      0);
end;
{$ENDIF}

procedure TFuenteYHorario.AfterInstantiation;
begin
  inherited AfterInstantiation;

  // obligo que los horarios esté ordenados.
  QuickSort_Creciente( horario_HABIL);
  QuickSort_Creciente( horario_SEMI_FERIADO );
  QuickSort_Creciente( horario_FERIADO );
end;


//-----------------------------------
// Métodos de TFichaFuenteSelector_horario
//===================================

constructor TFichaFuenteSelector_horario.Create(capa: integer; fecha: TFecha;
  periodicidad: TPeriodicidad; xusarTipoDeDia: boolean;
  fuentesYHorarios: TListaDeCosas;
  ValorPorDefecto_HABIL, ValorPorDefecto_SEMI_FERIADO, ValorPorDefecto_FERIADO: NReal);

begin
  inherited Create(capa, fecha, periodicidad);
  self.usarTipoDeDia := xusarTipoDeDia;
  self.FuentesYHorarios := fuentesYHorarios;
  self.ValorPorDefecto_HABIL := valorPorDefecto_HABIL;
  self.ValorPorDefecto_SEMI_FERIADO := valorPorDefecto_SEMI_FERIADO;
  self.ValorPorDefecto_FERIADO := valorPorDefecto_FERIADO;
end;

{$IFNDEF MIGRACION_PERSISTENCIA}
constructor TFichaFuenteSelector_horario.Create_ReadFromText(f: TArchiTexto);
begin
  inherited Create_ReadFromText(f);
  f.IniciarLecturaRetrasada;
  f.rd('usarTipoDeDia', usarTipoDeDia);
  f.rd('FuentesYHorarios', TCosa(FuentesYHorarios));
  f.rd('ValorPorDefecto_HABIL', ValorPorDefecto_HABIL);
  f.rd('ValorPorDefecto_SEMI_FERIADO', ValorPorDefecto_SEMI_FERIADO);
  f.rd('ValorPorDefecto_FERIADO', ValorPorDefecto_FERIADO);
  f.EjecutarLectura;
end;

procedure TFichaFuenteSelector_horario.WriteToText(f: TArchiTexto);
begin
  inherited WriteToText(f);
  f.wr('usarTipoDeDia', usarTipoDeDia);
  f.wr('FuentesYHorarios', TCosa(FuentesYHorarios));
  f.wr('ValorPorDefecto_HABIL', ValorPorDefecto_HABIL);
  f.wr('ValorPorDefecto_SEMI_FERIADO', ValorPorDefecto_SEMI_FERIADO);
  f.wr('ValorPorDefecto_FERIADO', ValorPorDefecto_FERIADO);
end;
{$ENDIF}

class function TFichaFuenteSelector_horario.DescClase: string;
begin
  Result := rsFichaDe + ' ' + rsFuenteSelector_horario;
end;

function TFichaFuenteSelector_horario.infoAd_: string;
begin
  Result := 'nHorarios: ' + IntToStr(FuentesYHorarios.Count);
end;

{$IFDEF MIGRACION_PERSISTENCIA}
class function TFichaFuenteSelector_horario.CreateDataColumnList(xClaseDeCosa: TClaseDeCosa; xVersion: Integer): TDataColumnListOfCosa;
begin
  Result:=inherited CreateDataColumnList(xClaseDeCosa, xVersion);
  Result.AddBooleanColumn('usarTipoDeDia',                true, 0);
  Result.AddCosaColumn   ('FuentesYHorarios',                   0);
  Result.AddDoubleColumn ('ValorPorDefecto_HABIL',        0.0,  0);
  Result.AddDoubleColumn ('ValorPorDefecto_SEMI_FERIADO', 0.0,  0);
  Result.AddDoubleColumn ('ValorPorDefecto_FERIADO',      0.0,  0);
end;
{$ENDIF}

//------------------------------
// Métodos de TFuenteSelector
//==============================

constructor TFuenteSelector_horario.Create(capa: integer; nombre: string; lpd: TFichasLPD);
begin
  inherited Create(capa, nombre, 0, True, lpd);
end;

class function TFuenteSelector_horario.TipoFichaFuente: TClaseDeFichaLPD;
begin
  Result := TFichaFuenteSelector_horario;
end;

{function TFuenteSelector_horario.InfoAd : String;
begin
  result:= 'Selector horario';
end;}

class function TFuenteSelector_horario.DescClase: string;
begin
  Result := rsFuenteSelector_horario;
end;

procedure TFuenteSelector_horario.RegistrarParametrosDinamicos;
begin
  //Se llama solo despues de preparar memoria que ya expande las fichas
  //no hace falta volver a hacerlo y es un proceso caro si hay periodicidad
  //   lpd.expandirFichas(globs);
  lpd.RegistrarFichasAActualizar(Self, globs.ActualizadorLPD, @pa,
    nil, procCambioFichaSelector_horario);
end;

procedure TFuenteSelector_horario.PrepararPaso_ps;
var
  k, n: integer;
  h: integer;
  pfh: TFuenteYHorario;
  tipoDelDia: TTipoDeDia;
  buscando: boolean;

begin
  k := 0;
  n := pa.FuentesYHorarios.Count;
  h := globs.HoraDeInicioDelPaso;

  // si está marcado usar el tipo de día, fijamos el tipo de dia de acuerdo
  // al inicio del paso. Si no, imponemos DIA_HABIL
  if pa.usarTipoDeDia then
    tipoDelDia := globs.TipoDeDiaInicioDelPaso
  else
    tipoDelDia := DIA_HABIL;

  // buscamos el primer filtro horario que contenga la hora de inicio del paso
  buscando := True;
  for k := 0 to n - 1 do
  begin
    pfh := pa.FuentesYHorarios.items[k] as TFuenteYHorario;

    if pfh.en_horario(h, tipoDelDia) then
    begin
      Bornera[0] := pfh.fuente.Bornera[pfh.idBorne];
      buscando := False;
      break;
    end;
  end;

  // si no se activó ningún filtro ponemos el valor por defecto
  if buscando then
    if tipoDelDia = DIA_HABIL then
      Bornera[0] := pa.ValorPorDefecto_HABIL
    else if tipoDelDia = DIA_SEMI_FERIADO then
      Bornera[0] := pa.ValorPorDefecto_SEMI_FERIADO
    else
      Bornera[0] := pa.ValorPorDefecto_FERIADO;
end;

procedure TFuenteSelector_horario.SortearEntradaRB(var aRB: NReal);
begin
  aRB := 0;
end;

procedure TFuenteSelector_horario.ValorEsperadoEntradaRB(var aRB: Nreal);
begin
  aRB := 0;
end;


function TFuenteSelector_horario.referenciaFuente(fuente: TFuenteAleatoria): boolean;
var
  k, n: integer;
  pfh: TFuenteYHorario;
  ficha_pd: TFichaFuenteSelector_horario;
  ificha: integer;
  buscando: Boolean;
begin
  k := 0;
  buscando:= true;
  while ificha < lpd.Count do
  begin
    ficha_pd:= lpd[ificha] as TFichaFuenteSelector_horario;
    n := ficha_pd.FuentesYHorarios.Count;
    for k := 0 to n - 1 do
    begin
      pfh := ficha_pd.FuentesYHorarios.items[k] as TFuenteYHorario;
      if pfh.fuente = fuente then
      begin
        buscando:= false;
        break;
      end;
    end;
    inc( ificha );
  end;
  Result := not buscando;
end;


procedure procCambioFichaSelector_horario(fuente: TCosa);
begin
  cambioFichaPDFuenteAleatoria(fuente);
end;

procedure AlInicio;
begin
  registrarClaseDeCosa(TFuenteYHorario.ClassName, TFuenteYHorario);
  registrarClaseDeCosa(TFuenteSelector_horario.ClassName, TFuenteSelector_horario);
  registrarClaseDeCosa(TFichaFuenteSelector_horario.ClassName,
    TFichaFuenteSelector_horario);
end;

procedure AlFinal;
begin
end;

end.
