unit usvgcanvas;
//mirar ADME/POSTOPERATIVO/utestsvg

{$mode delphi}

interface

uses
  Classes, types, SysUtils
  , DOM  , XMLWrite, FileUtil
  //, htmlEntity
  ;

type

TColor = cardinal;
TSVGPenStyle = (psSolid, psDash, psDot, psDashDot, psDashDotDot, psinsideFrame, psPattern,psClear);
TSVGBrushStyle =  (bsSolid, bsClear, bsHorizontal, bsVertical, bsFDiagonal,
                   bsBDiagonal, bsCross, bsDiagCross, bsImage, bsPattern);

TFPJPEGCompressionQuality = 1..100;   // 100 = best quality, 25 = pretty awful


const
  CFont_Alto = 2;//12; // ajustar
  CFont_Ancho = 0.5;//8; // ajustar

  // The following colors match the predefined Delphi Colors

  // standard colors
  clBlack   = TColor($000000);
  clMaroon  = TColor($000080);
  clGreen   = TColor($008000);
  clOlive   = TColor($008080);
  clNavy    = TColor($800000);
  clPurple  = TColor($800080);
  clTeal    = TColor($808000);
  clGray    = TColor($808080);
  clSilver  = TColor($C0C0C0);
  clRed     = TColor($0000FF);
  clLime    = TColor($00FF00);
  clYellow  = TColor($00FFFF);
  clBlue    = TColor($FF0000);
  clFuchsia = TColor($FF00FF);
  clAqua    = TColor($FFFF00);
  clLtGray  = TColor($C0C0C0); // clSilver alias
  clDkGray  = TColor($808080); // clGray alias
  clWhite   = TColor($FFFFFF);
  StandardColorsCount = 16;

  // extended colors
  clMoneyGreen = TColor($C0DCC0);
  clSkyBlue    = TColor($F0CAA6);
  clCream      = TColor($F0FBFF);
  clMedGray    = TColor($A4A0A0);
  ExtendedColorCount = 4;

  // special colors
  clNone    = TColor($1FFFFFFF);
  clDefault = TColor($20000000);

type

{ TSVGPen }

TSVGPen = class
  color: TColor;
  width: integer;
  style: TSVGPenStyle;
  constructor create;
end;

{ TSVGBrush }

TSVGBrush = class
  color: TColor;
  style: TSVGBrushStyle;
  constructor create;
end;

{ TSVGFont }

TSVGFont = class
  size: integer;
  orientation: integer;
  color: Tcolor;
  constructor create;
end;

{ TSVGCanvas }

TSVGCanvas = class
private
  rClipRect: TRect;
  clipRect_id: string;
  clipRect_cnt: integer;

  xdoc: TXMLDocument;
  NodoSVG: TDOMElement;
public
  width, height: integer;
  pen: TSVGPen;
  brush: TSVGBrush;
  Font: TSVGFont;
  penPos: TPoint;
  clipping: boolean;



  constructor Create( width, height: integer );
  procedure ClipRect(x1,y1,x2,y2:integer);
  procedure Free;
  procedure wr( s: string );
  procedure Line( x1, y1, x2, y2: integer );
  procedure Ellipse( x1, y1, x2, y2: integer );
  procedure Rectangle( x1, y1, x2, y2: integer );
  function  FillRect(  x1, y1, x2, y2: integer ): integer;
  procedure Polygon( r: array of TPoint );
  procedure MoveTo(x, y: integer );
  procedure LineTo(x, y: integer);
  function TextExtent(const Text: string): TSize;
  function TextHeight(s: string): integer;
  function TextWidth(s: string): integer;
  procedure TextOut(x, y: integer; texto: string );

  procedure WriteToArchi( archi: string );

  //function GetClipRect: TRect;
  //function GetClipRect: int64;
  //procedure SetClipRect( id: int64 );
  //property ClipRect: TRect read GetClipRect write SetClipRect;
  //property ClipRect: int64 read GetClipRect write SetClipRect;

end;

function RGBToColor(R, G, B: Byte): TColor;

function ColorToRGB(color:TColor): integer;

implementation

{ TSVGPen }

constructor TSVGPen.create;
begin
  inherited create;
  color:=clBlack;
  style:=psSolid;
  width:=1;

end;

{ TSVGBrush }

constructor TSVGBrush.create;
begin
  inherited create;
  color:=clBlack;
  style:=bsSolid;
end;

{ TSVGFont }

constructor TSVGFont.create;
begin
  inherited create;
  size:=12;
  orientation:=0;
  color:= clBlack;
end;



//function TSVGCanvas.GetClipRect: TRect;
//begin
// result:= rClipRect;
//end;

//procedure TSVGCanvas.SetClipRect(id: int64);
//begin
// clipRect_cnt:=id;
//
// //rchSetClipRect
// //rClipRect:= r;
// //inc( clipRect_cnt );
//end;
//

//procedure TSVGCanvas.SetClipRect(AValue: TRect);
//  var
//    NodoRect: TDOMElement;
//    NodoClip: TDOMElement;
//    ancho: Int64;
//    alto: Int64;
//  begin
//
//    NodoClip := xdoc.CreateElement('clipPath');
//    Result := clipRect_cnt + 1 ;
//    TDOMElement(NodoClip).SetAttribute('id', 'Clip' + IntToStr(Result));       // crear los atributos del nodo padre
//
//    NodoRect := xdoc.CreateElement('rect');
//
//    ancho:= round(abs(x2-x1));
//    alto := round(abs(y2-y1));
//
//    TDOMElement(NodoRect).SetAttribute('x', IntToStr(x1));       // crear los atributos del nodo padre
//    TDOMElement(NodoRect).SetAttribute('y', IntToStr(y1));       // crear los atributos del nodo padre
//    TDOMElement(NodoRect).SetAttribute('width', IntToStr(ancho));       // crear los atributos del nodo padre
//    TDOMElement(NodoRect).SetAttribute('height', IntToStr(alto));       // crear los atributos del nodo padre
//
//    estilo := '; fill: #' + IntToHex(clWhite, 6) + ' ;';
//    TDOMElement(NodoRect).SetAttribute('style', estilo);       // crear los atributos del nodo padre
//
//    NodoClip.AppendChild(NodoRect);
//    NodoSVG.Appendchild(NodoClip);                          // guardar nodo padre
//
//end;
//

constructor TSVGCanvas.Create( width, height: integer );
begin
  inherited create;
  xdoc := TXMLDocument.Create;

  //creo el documento
   NodoSVG := xdoc.CreateElement('svg');               //crear el nodo SVG
   TDOMElement(NodoSVG).SetAttribute('xmlns', 'http://www.w3.org/2000/svg');  // crear los atributos del nodo padre
   TDOMElement(NodoSVG).SetAttribute('width', IntToStr(width));  // crear los atributos del nodo padre
   TDOMElement(NodoSVG).SetAttribute('height', IntToStr(height));  // crear los atributos del nodo padre
   xdoc.Appendchild(NodoSVG);                          // guardar nodo raíz

   clipRect_cnt := 0;

  //rchCreate
  //inherited Create;
  //self.width:= width;
  //self.height:= height;
  pen:= TSVGPen.Create;
  brush:= TSVGBrush.Create;
  Font:= TSVGFont.Create;
  //clipRect_cnt:= 0;
end;

procedure TSVGCanvas.ClipRect(x1, y1, x2, y2:integer);
  var
    NodoRect: TDOMElement;
    NodoClip: TDOMElement;
    ancho: Int64;
    alto: Int64;
    id: Integer;
    estilo: String;
  begin
    clipping:= true;
    NodoClip := xdoc.CreateElement('clipPath');
    clipRect_cnt:= clipRect_cnt +1;
    id := clipRect_cnt ;
    TDOMElement(NodoClip).SetAttribute('id', 'Clip' + IntToStr(id));       // crear los atributos del nodo padre

    NodoRect := xdoc.CreateElement('rect');

    ancho:= round(abs(x2-x1));
    alto := round(abs(y2-y1));

    TDOMElement(NodoRect).SetAttribute('x', IntToStr(x1));       // crear los atributos del nodo padre
    TDOMElement(NodoRect).SetAttribute('y', IntToStr(y1));       // crear los atributos del nodo padre
    TDOMElement(NodoRect).SetAttribute('width', IntToStr(ancho));       // crear los atributos del nodo padre
    TDOMElement(NodoRect).SetAttribute('height', IntToStr(alto));       // crear los atributos del nodo padre

    estilo := '; fill: #' + IntToHex(clWhite, 6) + ' ;';
    TDOMElement(NodoRect).SetAttribute('style', estilo);       // crear los atributos del nodo padre

    NodoClip.AppendChild(NodoRect);
    NodoSVG.Appendchild(NodoClip);                          // guardar nodo padre

end;

procedure TSVGCanvas.Free;
begin
  pen.Free;
  brush.Free;
  Font.Free;
  inherited Free;
end;

procedure TSVGCanvas.wr( s: string );
begin
    { TODO : Implementar salida.
 }
end;


procedure TSVGCanvas.Line( x1, y1, x2, y2: integer );
var
  NodoLine: TDOMElement;
  estilo: String;
begin
  NodoLine := xdoc.CreateElement('line');

  TDOMElement(NodoLine).SetAttribute('x1', IntToStr(x1));       // crear los atributos del nodo padre
  TDOMElement(NodoLine).SetAttribute('y1', IntToStr(y1));       // crear los atributos del nodo padre
  TDOMElement(NodoLine).SetAttribute('x2', IntToStr(x2));       // crear los atributos del nodo padre
  TDOMElement(NodoLine).SetAttribute('y2', IntToStr(y2));       // crear los atributos del nodo padre
  estilo := '; fill: #' + IntToStr(ColorToRGB(pen.color)) + ' ; stroke: #'+IntToHex(ColorToRGB(pen.color),6);
  if clipping then
    estilo := estilo + '; clip-path: url(#Clip' + IntToStr(clipRect_cnt) + ') ;';
  TDOMElement(NodoLine).SetAttribute('style', estilo);       // crear los atributos del nodo padre
  NodoSVG.Appendchild(NodoLine);                          // guardar nodo padre

  //rchLine
  // http://www.w3schools.com/svg/svg_line.asp
  //wr( '<line x1="0" y1="0" x2="200" y2="200" style="stroke:rgb(255,0,0);stroke-width:2" />' );
end;

procedure TSVGCanvas.Ellipse( x1, y1, x2, y2: integer );
var
  NodoElip: TDOMElement;
  ycentro, xcentro, xradio, yradio: Int64;
  estilo: string;
begin
   NodoElip := xdoc.CreateElement('ellipse');

   xcentro:=round((x1+x2)/2);
   ycentro:=round((y1+y2)/2);
   xradio :=round(abs(x2-x1)/2);
   yradio :=round(abs(x2-x1)/2);

   TDOMElement(NodoElip).SetAttribute('cx', IntToStr(xcentro));       // crear los atributos del nodo padre
   TDOMElement(NodoElip).SetAttribute('cy', IntToStr(ycentro));       // crear los atributos del nodo padre
   TDOMElement(NodoElip).SetAttribute('rx', IntToStr(xradio));       // crear los atributos del nodo padre
   TDOMElement(NodoElip).SetAttribute('ry', IntToStr(yradio));       // crear los atributos del nodo padre
   estilo:= '; fill: #' + IntToHex(ColorToRGB(brush.color),6) + ' ; stroke: #'+ IntToHex(ColorToRGB(pen.color),6)+' ; stroke-width: '+inttostr(pen.width) +' ;';
   if clipping then
     estilo := estilo + '; clip-path: url(#Clip' + IntToStr(clipRect_cnt) + ') ;';
   TDOMElement(NodoElip).SetAttribute('style', estilo);       // crear los atributos del nodo padre

   NodoSVG.Appendchild(NodoElip);                          // guardar nodo padre

  //rchEllipse
  //wr( '<ellipse cx="200" cy="80" rx="100" ry="50" style="fill:yellow;stroke:purple;stroke-width:2" /> ');
end;

procedure TSVGCanvas.Rectangle( x1, y1, x2, y2: integer );
var
oldC: TColor;
begin
   //oldC:=brush.color;
   //brush.color:=clWhite;
   FillRect(x1,y1,x2,y2);
   //brush.color:=oldC;

    //rchRectangle
    //http://www.w3schools.com/svg/svg_rect.asp
    //wr( '<rect width="300" height="100" style="fill:rgb(0,0,255);stroke-width:3;stroke:rgb(0,0,0)" />');
end;

function TSVGCanvas.FillRect(x1, y1, x2, y2: integer): integer;
var
  NodoRect: TDOMElement;
  alto: Int64;
  ancho: Int64;
  estilo: String;

begin
  NodoRect := xdoc.CreateElement('rect');

  ancho:= round(abs(x2-x1));
  alto := round(abs(y2-y1));

  TDOMElement(NodoRect).SetAttribute('x', IntToStr(x1));       // crear los atributos del nodo padre
  TDOMElement(NodoRect).SetAttribute('y', IntToStr(y1{y1-alto}));       // crear los atributos del nodo padre
  TDOMElement(NodoRect).SetAttribute('width', IntToStr(ancho));       // crear los atributos del nodo padre
  TDOMElement(NodoRect).SetAttribute('height', IntToStr(alto));       // crear los atributos del nodo padre

  estilo := '; fill: #' + IntToHex(ColorToRGB(brush.color),6) + ' ; stroke: #'+ IntToHex(ColorToRGB(pen.color),6)+' ; stroke-width: '+inttostr(pen.width) +' ;';
  if clipping then
    estilo := estilo + '; clip-path: url(#Clip' + IntToStr(clipRect_cnt) + ') ;';
  TDOMElement(NodoRect).SetAttribute('style', estilo);       // crear los atributos del nodo padre

  NodoSVG.Appendchild(NodoRect);                          // guardar nodo padre

    //rchFillRect
    //http://www.w3schools.com/svg/svg_rect.asp
    //wr( '<rect width="300" height="100" style="fill:rgb(0,0,255);stroke-width:3;stroke:rgb(0,0,0)" />');
end;


procedure TSVGCanvas.Polygon( r: array of TPoint );
var
  NodoPoly: TDOMElement;
  k: Integer;
  puntos: String;
  estilo: String;
begin
  NodoPoly := xdoc.CreateElement('polygon');

  puntos:='';
  for k:=0 to high(r) do
    begin
      puntos := puntos + ' ' +IntToStr(r[k].x)+','+ IntToStr(r[k].y);
    end;

  TDOMElement(NodoPoly).SetAttribute('points', puntos);       // crear los atributos del nodo padre
  estilo:= '; fill: #' + IntToHex(ColorToRGB(brush.color),6) + ' ; stroke: #'+ IntToHex(ColorToRGB(pen.color),6)+' ; stroke-width: '+inttostr(pen.width) +' ;';
  if clipping then
    estilo := estilo + '; clip-path: url(#Clip' + IntToStr(clipRect_cnt) + ') ;';
  TDOMElement(NodoPoly).SetAttribute('style', estilo);       // crear los atributos del nodo padre

  NodoSVG.Appendchild(NodoPoly);                          // guardar nodo padre

  //rchPolygon
  // http://www.w3schools.com/svg/svg_polygon.asp
  //wr( '<polygon points="200,10 250,190 160,210" style="fill:lime;stroke:purple;stroke-width:1" />' );
end;


procedure TSVGCanvas.MoveTo(x, y: integer );
begin
  penPos.x:= x;
  penPos.y:= y;
end;

procedure TSVGCanvas.LineTo(x, y: integer );
begin
  line( penPos.x, penPos.y, x, y );
  MoveTo( x, y );
end;

function TSVGCanvas.TextExtent(const Text: string): TSize;
begin
  Result.cx:=TextHeight(text);
  Result.cy:=TextWidth(text);
end;


function TSVGCanvas.TextHeight( s: string ): integer;
begin
  result:= round(font.size* CFont_Alto);
end;

function TSVGCanvas.TextWidth(s: string): integer;
begin
  result:= round(font.size* CFont_Ancho * length( s ));
end;


procedure TSVGCanvas.TextOut(x, y: integer; texto: string );
var
  NodoText: TDOMElement;
  NodoCont: TDOMText;
  NodoG: TDOMElement;
  estilo: String;
  stext: String;
begin
  NodoText := xdoc.CreateElement('text');

  TDOMElement(NodoText).SetAttribute('x', IntToStr(x));       // crear los atributos del nodo padre
  TDOMElement(NodoText).SetAttribute('y', IntToStr(y));       // crear los atributos del nodo padre
  TDOMElement(NodoText).SetAttribute('transform', 'rotate( -' + IntToStr( round(font.orientation/10)) + ' ' + IntToStr(x) + ',' + IntToStr(y) + ')');       // crear los atributos del nodo padre
  estilo := '; fill: #' + IntToHex(ColorToRGB(font.color),6) + ' ;';
  TDOMElement(NodoText).SetAttribute('style', estilo);       // crear los atributos del nodo padre

  stext:= UTF8ToSys(texto);
  NodoText.TextContent:= stext;

  if clipping then
  begin
    NodoG := xdoc.CreateElement('g');
    TDOMElement(NodoG).SetAttribute('style', '; clip-path: url(#Clip' + IntToStr(clipRect_cnt) + ') ;');
    NodoG.Appendchild(NodoText);
    NodoSVG.Appendchild(NodoG);
    exit;
  end
  else
    NodoSVG.Appendchild(NodoText);                          // guardar nodo padre
  //NodoSVG.ChildNodes.Item[0].AppendChild(NodoText);       // insertar el nodo hijo en el correspondiente nodo padre

  //rchTextOut
  // http://www.w3schools.com/svg/svg_text.asp
  //wr( '<text x="0" y="15" fill="red" transform="rotate(30 20,40)">I love SVG</text>' );
end;

procedure TSVGCanvas.WriteToArchi( archi: string );
begin
  writeXMLFile(self.xdoc, archi); // escribir el XML
end;

//function TSVGCanvas.GetClipRect: int64;
//begin
//     result:=clipRect_cnt;
//end;

function RGBToColor(R, G, B: Byte): TColor;
begin
  Result := (B shl 16) or (G shl 8) or R;
end;

function ColorToRGB(color:TColor): integer;
var
  aux: TColor;
  red, green, blue: Integer;
begin
  red := (color and $000000ff) shl 16 ;//*65536;
  green:=(color and $0000ff00);
  blue:= (color and $00ff0000) shr 16;//div 65536;
  result:= red + green + blue;

end;

end.

