unit uReferenciaMonitor;
  {$MODE Delphi}

interface

uses
  uMonitores, uCosa, uCosaConNombre, uVarDefs, SysUtils, Classes, xMatDefs,
  uSalasDeJuego, uEventosOptSim, uConversions;

resourcestring
  mesNoSeEncuentraActor = 'No se encuentra el actor';
  mesNoSeEncuentraLaVariable = 'No se encuentra la variable';
  mesEnElActor = ' en el actor ';

type
  TClasesDeMonitores = class of TMonitores;
  AOfVarDef = array of TClasesDeVarDefs;

  TClaseReferenciaMonitor = class of TReferenciaMonitor;

  //Lanzada por el resolver de TReferenciaDefVar si no encuentra la Cosa o la
  //variable referenciada
  EReferenciaDefVarException = class(Exception)
  public
    constructor Create(nombreCosa, claseCosa: string); overload;//No se encuentra la cosa
    constructor Create(nombreCosa, claseCosa, nombreVar: string);
      overload; //No se encuentra la variable
  end;

  //Lanzada por crearMonitorDeVariable monitor si alguna variable no se pudo resolver
  EMonitorException = class(Exception)
  public
    cantVarsResueltas: integer;
    constructor Create(const msg: string; cantVarsResueltas: integer);
  end;

  { TReferenciaDefVar }

  TReferenciaDefVar = class(TCosa)
  public
    nombreCosa, claseCosa, nombreVar: string;

    //    indice : Integer; //-1 si la variable no es un vector
    //-2 si es un vector pero lo quiero visualizar como tal
    //0..n si quiero acceder al elemento indice del vector
    constructor Create(capa: integer; xnombreCosa, xclaseCosa, xnombreVar: string);

    {$IFNDEF MIGRACION_PERSISTENCIA}
    constructor Create_ReadFromText(f: TArchiTexto); override;
    procedure WriteToText(f: TArchiTexto); override;
    {$ENDIF}

    function ClaseNombre: string;
    function resolver(cosasMonitoreables: TListaDeCosasConNombre): TVarDef;

    procedure Free; override;

    {$IFDEF MIGRACION_PERSISTENCIA}
    class function CreateDataColumnList(xClaseDeCosa:TClaseDeCosa; xVersion: Integer=-2
      ): TDataColumnListOfCosa; override;
    published
      property _nombreCosa: string read nombreCosa write nombreCosa;
      property _claseCosa: string  read claseCosa  write claseCosa;
      property _nombreVar: string  read nombreVar  write nombreVar;
    {$ENDIF}
  end;

  TDAOfTReferenciaDefVar = array of TReferenciaDefVar;

  { TReferenciaMonitor }

  TReferenciaMonitor = class(TCosaConNombre)//EL nombre sera el nombre del monitor
  public
    Enabled: boolean;
    eventos: TDAOfEventoProc;

    monitor: TMonitores;
    posEnSala: integer;

    constructor Create(capa: integer; const xNombreMon: string;
      xEventosProcs: TDAOfEventoProc); reintroduce;

    {$IFNDEF MIGRACION_PERSISTENCIA}
    constructor Create_ReadFromText(f: TArchiTexto); override;
    procedure WriteToText(f: TArchiTexto); override;
    {$ENDIF}

    procedure AfterInstantiation;override;



    //Busca las variables que monitorea en la sala, las prepara y crea el
    //monitor para ellas asignandolo en monitor
    procedure crearMonitorDeVariables(cosasMonitoreables: TListaDeCosasConNombre);
      virtual; abstract;

    function monitoreaA(cosa: TCosaConNombre): boolean; virtual; abstract;

    //Retorna true si el cambio en la cosa afecta a la referencia al monitor
    function notificarCambioCosa(cosaOrig, cosaNueva: TCosaConNombre): boolean;
      virtual; abstract;

    //pos debe ser la posición en la array de la TSalaMonitoreada en que se encuentra
    //el monitor. Sirve para comunicarse entre el monitor y la sala
    procedure notificarPosEnManejador(pos: integer); virtual;

    //Filtra las cosas que no son monitoreables por el monitor. El mismo monitor,
    //otros monitores que lo monitoreen y aquellas que no publican variables
    //monitoreables por el tipo de monitor
    function cosasMonitoreables(cosasConVariablesPublicadas:
      TListaDeCosasConNombre{of TReferenciaMonitor}): TListaDeCosasConNombre;

    //Filtra las cosas que no son monitoreables por la clase de monitor.
    //Aquellas que no publican variables monitoreables por el tipo de monitor
    class function cosasMonitoreablesDeClase(cosasConVariablesPublicadas:
      TListaDeCosasConNombre): TListaDeCosasConNombre;

    //Retorna la lista de nombres de las variables de la cosa que son monitoreables
    //por la clase de referencia
    class function nombresVarsMonitoreables(cosa: TCosaConNombre): TStrings;

    class function tiposMonitoreables: AOfVarDef; virtual; abstract;
    class function puedeMonitorearTipo(varDef: TVarDef): boolean;
    function getReferenciasDefVars: TDAOfTReferenciaDefVar; virtual; abstract;

    function esDeSimulacion: boolean;
    function esDeOptimizacion: boolean;

    procedure Free; override;

    {$IFDEF MIGRACION_PERSISTENCIA}
    class function CreateDataColumnList(xClaseDeCosa:TClaseDeCosa; xVersion: Integer=-2): TDataColumnListOfCosa; override;
  published
    property _Enabled: boolean         read Enabled write Enabled;
    property _eventos: TDAOfEventoProc read eventos write eventos;
    {$ENDIF}
  end;

  { TReferenciaMonUniVar }

  TReferenciaMonUniVar = class(TReferenciaMonitor)
  public
    defVar: TReferenciaDefVar;

    //      indice : Integer; //-1 si la variable no es un vector
    //-2 si es un vector pero lo quiero tratar como tal
    //0..n si quiero acceder al elemento indice del vector
    constructor Create(capa: integer; xnombreMon, xnombreCosa, xclaseCosa, xnombreVar: string; xEventosProcs: TDAOfEventoProc);

    {$IFNDEF MIGRACION_PERSISTENCIA}
    constructor Create_ReadFromText(f: TArchiTexto); override;
    procedure WriteToText(f: TArchiTexto); override;
    {$ENDIF}

    {$IFDEF MIGRACION_PERSISTENCIA}
    class function CreateDataColumnList(xClaseDeCosa:TClaseDeCosa; xVersion: Integer=-2): TDataColumnListOfCosa; override;
    {$ENDIF}

    function monitoreaA(cosa: TCosaConNombre): boolean; override;
    function notificarCambioCosa(cosaOrig, cosaNueva: TCosaConNombre): boolean; override;
    function getReferenciasDefVars: TDAOfTReferenciaDefVar; override;
    procedure Free; override;
  published
    {$IFDEF MIGRACION_PERSISTENCIA}
    property _defVar: TReferenciaDefVar read defVar write defVar;
    {$ENDIF}
  end;

  { TReferenciaMonMultiVar }

  TReferenciaMonMultiVar = class(TReferenciaMonitor)
  public
    (**************************************************************************)
    (*               A T R I B U T O S   P E R S I S T E N T E S              *)
    (**************************************************************************)

    defVars: TListaDeCosas {of TReferenciaDefVar};

   (**************************************************************************)

    //    indices : TDAofNInt;//-1 si la variable no es un vector
    //-2 si es un vector pero lo quiero tratar como tal
    //0..n si quiero acceder al elemento indice del vector
    {$IFDEF MIGRACION_PERSISTENCIA}
    class function CreateDataColumnList(xClaseDeCosa:TClaseDeCosa; xVersion: Integer=-2
      ): TDataColumnListOfCosa; override;
    {$ENDIF}
    constructor Create(capa: integer; xnombreMon: string;
      xEventosProcs: TDAOfEventoProc; defVars: TListaDeCosas);

    {$IFNDEF MIGRACION_PERSISTENCIA}
    constructor Create_ReadFromText(f: TArchiTexto); override;
    procedure WriteToText(f: TArchiTexto); override;
    {$ENDIF}

    function monitoreaA(cosa: TCosaConNombre): boolean; override;
    function notificarCambioCosa(cosaOrig, cosaNueva: TCosaConNombre): boolean; override;
    function getReferenciasDefVars: TDAOfTReferenciaDefVar; override;

    procedure Free; override;
  published
    {$IFDEF MIGRACION_PERSISTENCIA}
    property _defVars: TListaDeCosas read defVars write defVars;
    {$ENDIF}
  end;

procedure AlInicio;
procedure AlFinal;

implementation

//-------------------------------------
//Metodos de EReferenciaDefVarException
//=====================================

constructor EReferenciaDefVarException.Create(nombreCosa, claseCosa: string);
begin
  inherited Create(mesNoSeEncuentraActor + ' ' + claseCosa + ', ' +
    nombreCosa + '.' + #13);
end;

constructor EReferenciaDefVarException.Create(nombreCosa, claseCosa, nombreVar: string);
begin
  inherited Create(mesNoSeEncuentraLaVariable + ' ' + nombreVar +
    mesEnElActor + claseCosa + ', ' + nombreCosa + '.' + #13);
end;

//----------------------------
//Metodos de EMonitorException
//============================

constructor EMonitorException.Create(const msg: string; cantVarsResueltas: integer);
begin
  inherited Create(msg);
  self.cantVarsResueltas := cantVarsResueltas;
end;

//----------------------------
//Metodos de TReferenciaDefVar
//============================

constructor TReferenciaDefVar.Create(capa: integer;
  xnombreCosa, xclaseCosa, xnombreVar: string{; indice : Integer});
begin
  inherited Create(capa);
  nombreCosa := xnombreCosa;
  claseCosa := xclaseCosa;
  nombreVar := xnombreVar;
  //  self.indice := indice;
end;

{$IFNDEF MIGRACION_PERSISTENCIA}
constructor TReferenciaDefVar.Create_ReadFromText(f: TArchiTexto);
begin
  inherited Create_ReadFromText(f);
  f.IniciarLecturaRetrasada;
  f.rd('nombreCosa', nombreCosa);
  f.rd('claseCosa', claseCosa);
  f.rd('nombreVar', nombreVar);
  f.EjecutarLectura;
end;
{$ENDIF}

{$IFNDEF MIGRACION_PERSISTENCIA}
procedure TReferenciaDefVar.WriteToText(f: TArchiTexto);
begin
  inherited WriteToText(f);
  f.wr('nombreCosa', nombreCosa);
  f.wr('claseCosa', claseCosa);
  f.wr('nombreVar', nombreVar);
end;
{$ENDIF}

function TReferenciaDefVar.ClaseNombre: string;
begin
  Result := claseCosa + ', ' + nombreCosa;
end;

function TReferenciaDefVar.resolver(cosasMonitoreables: TListaDeCosasConNombre): TVarDef;
var
  res: TVarDef;
  ipos: integer;
begin
  if cosasMonitoreables.find(claseCosa, nombreCosa, ipos) then
  begin
    res := TCosaConNombre(cosasMonitoreables[ipos]).buscarVariable(nombreVar);
    if res <> nil then
      Result := res
    else
      raise EReferenciaDefVarException.Create(nombreCosa, claseCosa, nombreVar);
  end
  else
  begin
    raise EReferenciaDefVarException.Create(nombreCosa, claseCosa);
    Result := nil;
  end;
end;

procedure TReferenciaDefVar.Free;
begin
  SetLength(nombreCosa, 0);
  SetLength(claseCosa, 0);
  SetLength(nombreVar, 0);
  inherited Free;
end;

{$IFDEF MIGRACION_PERSISTENCIA}
class function TReferenciaDefVar.CreateDataColumnList(xClaseDeCosa: TClaseDeCosa; xVersion: Integer): TDataColumnListOfCosa;
begin
  Result:=inherited CreateDataColumnList(xClaseDeCosa,xVersion);
  Result.AddStringColumn('nombreCosa','');
  Result.AddStringColumn('claseCosa','');
  Result.AddStringColumn('nombreVar','');
end;
{$ENDIF}

//-----------------------------
//Metodos de TReferenciaMonitor
//=============================

{$IFDEF MIGRACION_PERSISTENCIA}
class function TReferenciaMonitor.CreateDataColumnList(
  xClaseDeCosa: TClaseDeCosa; xVersion: Integer): TDataColumnListOfCosa;
begin

  if (xVersion<>-2) then
    raise Exception.Create('Está intentando utilizar un archivo *.ese no soportado en la versión actual del sistema. \n\r' +
                           'MENSAJE INTERNO: \n\r' +
                           '---------------- \n\r' +
                           'TReferenciaMonitor.CreateDataColumnList: Es necesario cambiar la forma en que se representa el array de eventos en el archivo *.ese');

  Result:=inherited CreateDataColumnList(xClaseDeCosa, xVersion);
  Result.AddBooleanColumn('Enabled',false);
end;
{$ENDIF}

procedure TReferenciaMonitor.AfterInstantiation;
begin
  inherited AfterInstantiation;
  posEnSala := -1;
  monitor := nil;
end;

constructor TReferenciaMonitor.Create(capa: integer; const xNombreMon: string;
  xEventosProcs: TDAOfEventoProc);
begin
  inherited Create( capa, xNombreMon);
  eventos := xEventosProcs;
  Enabled := True;
  posEnSala := -1;
  pubvarlst := nil;
  monitor := nil;
end;

{$IFNDEF MIGRACION_PERSISTENCIA}
constructor TReferenciaMonitor.Create_ReadFromText(f: TArchiTexto);
var
  i, nEventos: integer;
  cadena: string;
begin
  inherited Create_ReadFromText(f);
  f.IniciarLecturaRetrasada;
  f.rd('enabled', Enabled);
  f.rd('nEventos', nEventos);
  f.EjecutarLectura;
  SetLength(eventos, nEventos);
  for i := 0 to high(eventos) do
  begin
    f.IniciarLecturaRetrasada;
    f.rd('cadena', cadena);
    f.rd('refProc', eventos[i].refProc);
    f.EjecutarLectura;
    eventos[i].evento := StrToEvento(cadena);
  end;
  monitor := nil;
  posEnSala := -1;
end;
{$ENDIF}

{$IFNDEF MIGRACION_PERSISTENCIA}
procedure TReferenciaMonitor.WriteToText(f: TArchiTexto);
var
  i, nEventos: integer;
begin
  inherited WriteToText(f);
  f.wr('enabled', Enabled);

  nEventos := length(eventos);
  f.wr('nEventos', nEventos);

  for i := 0 to high(eventos) do
  begin
    f.wr('cadena', EventoToStr(eventos[i].evento));
    f.wr('refProc', eventos[i].refProc);
  end;
end;
{$ENDIF}

procedure TReferenciaMonitor.notificarPosEnManejador(pos: integer);
begin
  posEnSala := pos;
end;

function TReferenciaMonitor.cosasMonitoreables(cosasConVariablesPublicadas:
  TListaDeCosasConNombre{of TReferenciaMonitor}): TListaDeCosasConNombre;
var
  i, j: integer;
  tieneTipoMonitoreable: boolean;
  cosa: TCosaConNombre;
begin
  //  resultado := cosasConVariablesPublicadas.Create_Clone as TListaDeCosasConNombre;

  //Saco a los monitores que me monitorean (y a mi mismo)
  cosasConVariablesPublicadas.Remove(self);
  for i := 0 to cosasConVariablesPublicadas.Count - 1 do
  begin
    tieneTipoMonitoreable := False;
    cosa := TCosaConNombre(cosasConVariablesPublicadas[i]);
    for j := 0 to cosa.pubvarlst.Count - 1 do
    begin
      if puedeMonitorearTipo(cosa.pubvarlst[j]) then
      begin
        tieneTipoMonitoreable := True;
        break;
      end;
    end;

    //Si no publica una variable que yo pueda monitorear o es un monitor que me
    //monitorea a mi lo saco de la lista de cosas que puedo monitorear
    if (not tieneTipoMonitoreable) or
      ((TCosaConNombre(cosasConVariablesPublicadas[i]) is TReferenciaMonitor) and
      (TReferenciaMonitor(cosasConVariablesPublicadas[i]).monitoreaA(self))) then
      cosasConVariablesPublicadas[i] := nil;
  end;
  cosasConVariablesPublicadas.Pack;
  Result := cosasConVariablesPublicadas;
end;

class function TReferenciaMonitor.cosasMonitoreablesDeClase(
  cosasConVariablesPublicadas: TListaDeCosasConNombre): TListaDeCosasConNombre;
var
  i, j: integer;
  tieneTipoMonitoreable: boolean;
  cosa: TCosaConNombre;
begin
  //  resultado := cosasConVariablesPublicadas.Create_Clone as TListaDeCosasConNombre;

  //Saco todas las cosas que no publican variables que yo pueda monitorear
  for i := 0 to cosasConVariablesPublicadas.Count - 1 do
  begin
    tieneTipoMonitoreable := False;
    cosa := TCosaConNombre(cosasConVariablesPublicadas[i]);
    for j := 0 to cosa.pubvarlst.Count - 1 do
    begin
      if puedeMonitorearTipo(cosa.pubvarlst[j]) then
      begin
        tieneTipoMonitoreable := True;
        break;
      end;
    end;
    if not tieneTipoMonitoreable then
      cosasConVariablesPublicadas[i] := nil;
  end;
  cosasConVariablesPublicadas.Pack;
  Result := cosasConVariablesPublicadas;
end;

class function TReferenciaMonitor.nombresVarsMonitoreables(cosa:
  TCosaConNombre): TStrings;
var
  i: integer;
  resultado: TStringList;
begin
  resultado := TStringList.Create;
  for i := 0 to cosa.pubvarlst.Count - 1 do
    if puedeMonitorearTipo(TVarDef(cosa.pubvarlst[i])) then
      resultado.Add(TVarDef(cosa.pubvarlst[i]).nombreVar);
  Result := resultado;
end;

function TReferenciaMonitor.esDeSimulacion: boolean;
var
  i: integer;
  resultado: boolean;
begin
  resultado := False;
  for  i := 0 to high(eventos) do
    if TipoDeEvento(eventos[i].evento) = 1 then
    begin
      resultado := True;
      break;
    end;
  Result := resultado;
end;

function TReferenciaMonitor.esDeOptimizacion: boolean;
var
  i: integer;
  resultado: boolean;
begin
  resultado := False;
  for  i := 0 to high(eventos) do
    if TipoDeEvento(eventos[i].evento) = 2 then
    begin
      resultado := True;
      break;
    end;
  Result := resultado;
end;

class function TReferenciaMonitor.puedeMonitorearTipo(varDef: TVarDef): boolean;
var
  i: integer;
  aux: AOfVarDef;
  resultado: boolean;
begin
  resultado := False;
  aux := Self.tiposMonitoreables;
  for i := 0 to high(aux) do
    if varDef is aux[i] then
    begin
      resultado := True;
      break;
    end;
  Result := resultado;
end;

procedure TReferenciaMonitor.Free;
begin
  if monitor <> nil then
    monitor.Free;
  SetLength(eventos, 0);
  inherited Free;
end;

//-------------------------------
//Metodos de TReferenciaMonUniVar
//===============================

constructor TReferenciaMonUniVar.Create( capa: integer; xnombreMon, xnombreCosa,
  xclaseCosa, xnombreVar: string;{ indice : Integer ; }xEventosProcs: TDAOfEventoProc);
begin
  inherited Create(capa, xnombreMon, xEventosProcs);
  defVar := TReferenciaDefVar.Create(capa, xnombreCosa, xclaseCosa, xnombreVar);
end;

{$IFNDEF MIGRACION_PERSISTENCIA}
constructor TReferenciaMonUniVar.Create_ReadFromText(f: TArchiTexto);
var
  nombreCosa: string;
  claseCosa: string;
  nombreVar: string;
begin
  if f.Version < 18 then
  begin
    inherited Create_ReadFromText(f);
    f.IniciarLecturaRetrasada;
    f.rd('nombreCosa', nombreCosa);
    f.rd('claseCosa', claseCosa);
    f.rd('nombreVar', nombreVar);
    f.EjecutarLectura;
    self.defVar := TReferenciaDefVar.Create(capa, nombreCosa, claseCosa, nombreVar);
  end
  else
  begin
    inherited Create_ReadFromText(f);
    f.IniciarLecturaRetrasada;
    f.rd('defVar', TCosa(defVar));
    f.EjecutarLectura;
  end;
end;
{$ENDIF}

{$IFNDEF MIGRACION_PERSISTENCIA}
procedure TReferenciaMonUniVar.WriteToText(f: TArchiTexto);
begin
  inherited WriteToText(f);
  f.wr('defVar', defVar);
end;
{$ENDIF}

{$IFDEF MIGRACION_PERSISTENCIA}
class function TReferenciaMonUniVar.CreateDataColumnList(xClaseDeCosa: TClaseDeCosa; xVersion: Integer): TDataColumnListOfCosa;
begin
  Result:=inherited CreateDataColumnList(xClaseDeCosa, xVersion);
  Result.AddCosaColumn('defVar', 18);
end;
{$ENDIF}

function TReferenciaMonUniVar.monitoreaA(cosa: TCosaConNombre): boolean;
begin
  Result := (defvar.nombreCosa = cosa.nombre) and (defvar.claseCosa = cosa.ClassName);
end;

function TReferenciaMonUniVar.notificarCambioCosa(cosaOrig, cosaNueva:
  TCosaConNombre): boolean;
begin
  if (defvar.nombreCosa = cosaOrig.nombre) and (defvar.claseCosa =
    cosaOrig.ClassName) then
  begin
    defvar.nombreCosa := cosaNueva.nombre;
    Result := True;
  end
  else
    Result := False;
end;

function TReferenciaMonUniVar.getReferenciasDefVars: TDAOfTReferenciaDefVar;
var
  res: TDAOfTReferenciaDefVar;
begin
  SetLength(res, 1);
  res[0] := defVar;
  Result := res;
end;

procedure TReferenciaMonUniVar.Free;
begin
  defVar.Free;
  inherited Free;
end;

//---------------------------------
//Metodos de TReferenciaMonMultiVar
//=================================

{$IFDEF MIGRACION_PERSISTENCIA}
class function TReferenciaMonMultiVar.CreateDataColumnList(
  xClaseDeCosa: TClaseDeCosa; xVersion: Integer): TDataColumnListOfCosa;
begin
  Result:=inherited CreateDataColumnList(xClaseDeCosa, xVersion);
  Result.AddCosaColumn('defVars');
end;
{$ENDIF}

constructor TReferenciaMonMultiVar.Create(capa: integer;xnombreMon: string;
  xEventosProcs: TDAOfEventoProc; defVars: TListaDeCosas);
begin
  inherited Create( capa, xnombreMon, xEventosProcs);
  self.defVars := defVars;
end;

{$IFNDEF MIGRACION_PERSISTENCIA}
constructor TReferenciaMonMultiVar.Create_ReadFromText(f: TArchiTexto);
begin
  inherited Create_ReadFromText(f);
  f.IniciarLecturaRetrasada;
  f.rd('defVars', TCosa(defVars));
  f.EjecutarLectura;
end;
{$ENDIF}

{$IFNDEF MIGRACION_PERSISTENCIA}
procedure TReferenciaMonMultiVar.WriteToText(f: TArchiTexto);
begin
  inherited WriteToText(f);
  f.wr('defVars', defVars);
end;
{$ENDIF}

function TReferenciaMonMultiVar.monitoreaA(cosa: TCosaConNombre): boolean;
var
  i: integer;
  encontre: boolean;
begin
  encontre := False;
  for  i := 0 to defVars.Count - 1 do
    if (TReferenciaDefVar(defVars[i]).nombreCosa = cosa.nombre) and
      (TReferenciaDefVar(defVars[i]).claseCosa = cosa.ClassName) then
    begin
      encontre := True;
      break;
    end;
  Result := encontre;
end;

function TReferenciaMonMultiVar.notificarCambioCosa(cosaOrig, cosaNueva:
  TCosaConNombre): boolean;
var
  i: integer;
  res: boolean;
begin
  res := False;
  for i := 0 to defVars.Count - 1 do
    if (TReferenciaDefVar(defVars[i]).nombreCosa = cosaOrig.nombre) and
      (TReferenciaDefVar(defVars[i]).claseCosa = cosaOrig.ClassName) then
    begin
      TReferenciaDefVar(defVars[i]).nombreCosa := cosaNueva.nombre;
      res := True;
    end;
  Result := res;
end;

function TReferenciaMonMultiVar.getReferenciasDefVars: TDAOfTReferenciaDefVar;
begin
  Result := TDAOfTReferenciaDefVar(defVars.toArray);
end;

procedure TReferenciaMonMultiVar.Free;
begin
  defVars.Free;
  inherited Free;
end;

procedure AlInicio;
begin
  registrarClaseDeCosa(TReferenciaDefVar.ClassName, TReferenciaDefVar);
end;

procedure AlFinal;
begin
end;

end.
